package ch.psi.daq.queryrest.query;

import java.util.Collection;
import java.util.List;
import java.util.Map.Entry;
import java.util.function.Supplier;
import java.util.stream.Stream;

import org.apache.commons.lang3.tuple.Triple;

import ch.psi.daq.common.tuple.Quadruple;
import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.events.ChannelConfiguration;
import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.domain.query.DAQConfigQuery;
import ch.psi.daq.domain.query.DAQConfigQueryElement;
import ch.psi.daq.domain.query.DAQQueries;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.backend.BackendQuery;
import ch.psi.daq.domain.query.channels.ChannelConfigurationsRequest;
import ch.psi.daq.domain.query.channels.ChannelConfigurationsResponse;
import ch.psi.daq.domain.query.channels.ChannelsRequest;
import ch.psi.daq.domain.query.channels.ChannelsResponse;
import ch.psi.daq.domain.query.channels.LongHash;

public interface QueryManager {

   Stream<Backend> getBackends();

    /**
    * Provides the root path of the redirection (or <tt>null</tt> if no redirection is possible).
    *
    * @param channels The channels
    * @return String The root of the redirection or <tt>null</tt> for none.
    */
    String getRedirection(final Supplier<Collection<ChannelName>> channels);

   LongHash getChannelsHash();

   Stream<ChannelsResponse> getChannels(
         final ChannelsRequest request) throws Exception;

   LongHash getChannelConfigurationsHash();

   Stream<ChannelConfigurationsResponse> getChannelConfigurations(
         final ChannelConfigurationsRequest request)
         throws Exception;

   ChannelConfiguration getChannelConfiguration(
         final ChannelName channel) throws Exception;

   Entry<DAQConfigQueryElement, Stream<Triple<BackendQuery, ChannelName, ?>>> queryConfigs(
         final DAQConfigQuery query)
         throws Exception;

   List<Entry<DAQQueryElement, Stream<Quadruple<BackendQuery, ChannelName, ?, ?>>>> queryEvents(
         final DAQQueries queries)
         throws Exception;
}
