package ch.psi.daq.queryrest.query;

import java.util.Collection;
import java.util.Set;
import java.util.function.Function;
import java.util.function.Supplier;

import javax.annotation.PreDestroy;

import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.domain.query.backend.BackendQuery;
import ch.psi.daq.domain.query.backend.analyzer.BackendQueryAnalyzer;
import ch.psi.daq.domain.query.channels.BackendsChannelConfigurationCache;
import ch.psi.daq.query.config.QueryConfig;
import ch.psi.daq.queryrest.config.QueryRestConfig;

public class QueryManagerImpl extends AbstractQueryManager implements ApplicationContextAware {
   public static final String QUERY_SERVER_TYPE = "local";

   @SuppressWarnings("unchecked")
   @Override
   public void setApplicationContext(ApplicationContext context) throws BeansException {
      final Backend backend = context.getBean(DomainConfig.BEAN_NAME_BACKEND_DEFAULT, Backend.class);
      context = backend.getApplicationContext();

      final Set<Backend> activeBackends = context.getBean(DomainConfig.BEAN_NAME_BACKENDS_ACTIVE, Set.class);
      final BackendsChannelConfigurationCache channelsCache =
            context.getBean(QueryConfig.BEAN_NAME_HISTORIC_CHANNELS_CACHE, BackendsChannelConfigurationCache.class);
      final Function<BackendQuery, BackendQueryAnalyzer> queryAnalizerFactory =
            context.getBean(QueryRestConfig.BEAN_NAME_QUERY_ANALIZER_FACTORY, Function.class);

      init(activeBackends, channelsCache, queryAnalizerFactory);
   }

   @PreDestroy
   public void destroy() {}

   @Override
   public String getRedirection(final Supplier<Collection<ChannelName>> channels) {
      return null;
   }
}
