package ch.psi.daq.queryrest.response;

import java.io.OutputStream;

import javax.servlet.http.HttpServletResponse;

import org.springframework.context.ApplicationContext;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.core.JsonEncoding;

import ch.psi.daq.domain.query.DAQQueries;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.response.ResponseFormat;
import ch.psi.daq.domain.query.response.ResponseImpl;

public abstract class AbstractHTTPResponse extends ResponseImpl {

   public AbstractHTTPResponse(final ResponseFormat format) {
      super(format);
   }

   @JsonIgnore
   public abstract void validateQuery(final Object queryObj);

   @JsonIgnore
   public boolean useTableFormat(final Object queryObj) {
      if (queryObj instanceof DAQQueries) {
         final DAQQueries queries = (DAQQueries) queryObj;

         for (final DAQQueryElement query : queries) {
            if (query.getMapping() != null) {
               return true;
            }
         }
      } else if (queryObj instanceof DAQQueryElement) {
         return ((DAQQueryElement) queryObj).getMapping() != null;
      }

      return false;
   }

   @JsonIgnore
   public abstract <R> void respond(
         final ApplicationContext context,
         final HttpServletResponse httpResponse,
         final Object query, 
         final R result)
         throws Exception;

   /**
    * Configures the output stream and headers according to whether compression is wanted or not.
    * <p>
    * In order not to lose the information of the underlying type of data being transferred, the
    * Content-Type header stays the same but, if compressed, the content-encoding header will be set
    * accordingly.
    * 
    * see http://tools.ietf.org/html/rfc2616#section-14.11 and see
    * http://tools.ietf.org/html/rfc2616#section-3.5
    * 
    * @param httpResponse The HttpServletResponse
    * @param contentType The content type
    * @return OutputStream The OutputStream
    * @throws Exception Something goes wrong
    */
   @JsonIgnore
   protected OutputStream handleCompressionAndResponseHeaders(final HttpServletResponse httpResponse,
         final String contentType) throws Exception {
      OutputStream out = httpResponse.getOutputStream();

      httpResponse.setCharacterEncoding(JsonEncoding.UTF8.getJavaName());
      httpResponse.setContentType(contentType);

      httpResponse.addHeader("Content-Type", contentType);
      final String filename = "data." + this.getFileSuffix();
      httpResponse.addHeader("Content-Disposition", "attachment; filename=" + filename);

      if (this.isCompressed()) {
         httpResponse.addHeader("Content-Encoding", this.getCompression().toString());
         out = this.getCompression().wrapStream(out);
      }

      return out;
   }
}
