package ch.psi.daq.queryrest.response.csv;

import java.io.OutputStream;

import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;

import com.hazelcast.util.collection.ArrayUtils;

import ch.psi.daq.domain.FieldNames;
import ch.psi.daq.domain.query.DAQQueries;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.operation.AggregationType;
import ch.psi.daq.domain.query.operation.Compression;
import ch.psi.daq.domain.query.operation.EventField;
import ch.psi.daq.domain.query.response.ResponseFormat;
import ch.psi.daq.queryrest.response.AbstractHTTPResponse;

public class CSVHTTPResponse extends AbstractHTTPResponse {
   private static final Logger LOGGER = LoggerFactory.getLogger(CSVHTTPResponse.class);

   public static final String FORMAT = "csv";
   public static final String CONTENT_TYPE = "text/csv";

   public CSVHTTPResponse() {
      super(ResponseFormat.CSV);
   }

   public CSVHTTPResponse(final Compression compression) {
      this();
      setCompression(compression);
   }

   @Override
   public void validateQuery(final Object queryObj) {
      if (queryObj instanceof DAQQueries) {
         final DAQQueries queries = (DAQQueries) queryObj;
         for (final DAQQueryElement query : queries) {
            if (!(query.getAggregation() == null || AggregationType.value.equals(query.getAggregation()
                  .getAggregationType()))) {
               // We allow only no aggregation or value aggregation as
               // extrema: nested structure and not clear how to map it to one line
               // index: value is an array of Statistics whose size is not clear at initialization
               // time
               final String message =
                     "CSV export does not support '" + query.getAggregation().getAggregationType() + "'";
               LOGGER.warn(message);
               throw new IllegalArgumentException(message);
            }


            if (!ArrayUtils.contains(query.getRequest().getColumns(), FieldNames.FIELD_GLOBAL_TIME)) {
               query.addEventField(EventField.globalMillis);
            }
         }
      }
   }

   @Override
   public <R> void respond(
         final ApplicationContext context,
         final HttpServletResponse response,
         final Object query,
         final R result) throws Exception {
      final OutputStream out = handleCompressionAndResponseHeaders(response, CONTENT_TYPE);

      final CSVResponseStreamWriter streamWriter = context.getBean(CSVResponseStreamWriter.class);

      // write the response back to the client using java 8 streams
      streamWriter.respond(query, result, out, this);
   }
}
