package ch.psi.daq.queryrest.response.csv;

import java.util.Collections;
import java.util.Map;
import java.util.function.Function;

import ch.psi.daq.common.string.stringifier.Stringifier;
import ch.psi.daq.common.util.Arrays;
import ch.psi.data.collection.PrimitiveList;

public class QueryFieldStringifyer implements Stringifier {
   public static final String OPEN_BRACKET = "[";
   public static final String CLOSE_BRACKET = "]";

   private final Function<Object, Object> accessor;
   private final String nonValue;
   private final String arraySeparator;
   private final Map<Class<?>, Stringifier> stringifiers;

   public QueryFieldStringifyer(
         final Function<Object, Object> accessor,
         final String nonValue,
         final String arraySeparator,
         final Map<Class<?>, Stringifier> stringifiers) {
      this.accessor = accessor;
      this.nonValue = nonValue;
      this.arraySeparator = arraySeparator;
      if (stringifiers != null) {
         this.stringifiers = stringifiers;
      } else {
         this.stringifiers = Collections.emptyMap();
      }
   }

   protected Function<Object, Object> getAccessor() {
      return accessor;
   }

   protected String getNonValue() {
      return nonValue;
   }

   @SuppressWarnings("rawtypes")
   @Override
   public String toString(Object value) {
      if (value == null) {
         return nonValue;
      } else if (value instanceof PrimitiveList) {
         return toString((PrimitiveList) value, arraySeparator);
      } else if (value.getClass().isArray()) {
         return Arrays.toString(value, arraySeparator, stringifiers.get(value.getClass()));
      } else {
         final Stringifier stringifier = stringifiers.get(value.getClass());
         if (stringifier != null) {
            return stringifier.toString(value);
         } else {
            return value.toString();
         }
      }
   }

   public static String toString(PrimitiveList<?> value, String arraySeparator) {
      int size = value.size();
      if (size < 1) {
         return Arrays.NULL_STRING;
      } else if (size == 1) {
         // scalar
         return value.getAsString(0);
      } else {
         int iMax = size - 1;
         if (iMax == -1)
            return Arrays.EMPTY_ARRAY;

         StringBuilder b = new StringBuilder();
         b.append(OPEN_BRACKET);
         for (int i = 0;; i++) {
            b.append(value.getAsString(i));
            if (i == iMax)
               return b.append(CLOSE_BRACKET).toString();
            b.append(arraySeparator);
         }
      }
   }
}
