package ch.psi.daq.queryrest.response.formatter;

import java.io.OutputStream;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import com.fasterxml.jackson.core.JsonEncoding;
import com.fasterxml.jackson.core.JsonFactory;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;

import ch.psi.daq.domain.query.operation.ConfigField;
import ch.psi.daq.domain.query.operation.EventField;
import ch.psi.daq.domain.query.operation.QueryField;
import ch.psi.daq.queryrest.response.AbstractHTTPResponse;
import ch.psi.daq.queryrest.response.ResponseFormatter;

public class AnyResponseFormatter implements ResponseFormatter<Object>,
      ApplicationContextAware {
   private String eventFieldsBeanName;
   private String configFieldsBeanName;

   private Set<String> eventFields;
   private Set<String> configFields;

   public AnyResponseFormatter(final String eventFieldsBeanName, final String configFieldsBeanName) {
      this.eventFieldsBeanName = eventFieldsBeanName;
      this.configFieldsBeanName = configFieldsBeanName;
   }

   public AnyResponseFormatter(final Set<String> eventFields, final Set<String> configFields) {
      this.eventFields = eventFields;
      this.configFields = configFields;
   }

   @SuppressWarnings("unchecked")
   @Override
   public void setApplicationContext(ApplicationContext context) throws BeansException {
      if (eventFields == null) {
         final Set<EventField> defaultEventFields =
               context.getBean(eventFieldsBeanName, Set.class);
         this.eventFields =
               defaultEventFields.stream().map(QueryField::getName)
                     .collect(Collectors.toCollection(LinkedHashSet::new));
      }

      if (configFields == null) {
         final Set<ConfigField> defaultConfigFields =
               context.getBean(configFieldsBeanName, Set.class);
         this.configFields =
               defaultConfigFields.stream().map(QueryField::getName)
                     .collect(Collectors.toCollection(LinkedHashSet::new));
      }
   }

   public Set<String> getEventFields() {
      return Collections.unmodifiableSet(eventFields);
   }

   public Set<String> getConfigFields() {
      return Collections.unmodifiableSet(configFields);
   }

   @Override
   public void format(
         final JsonFactory factory,
         final ObjectMapper mapper,
         final Object result,
         final OutputStream out,
         final AbstractHTTPResponse response) throws Exception {
      final JsonGenerator generator = factory.createGenerator(out, JsonEncoding.UTF8);
      final ObjectWriter writer = DAQQueriesResponseFormatter.configureWriter(mapper, eventFields, configFields);

      try {
         writer.writeValue(generator, result);
      } finally {
         generator.flush();
         generator.close();
      }
   }
}
