package ch.psi.daq.queryrest.response.json;

import java.io.OutputStream;
import java.util.List;
import java.util.Map.Entry;
import java.util.stream.Stream;

import javax.servlet.ServletResponse;

import org.apache.commons.lang3.tuple.Triple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.core.JsonFactory;
import com.fasterxml.jackson.databind.ObjectMapper;

import ch.psi.daq.common.tuple.Quadruple;
import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.domain.query.DAQConfigQuery;
import ch.psi.daq.domain.query.DAQConfigQueryElement;
import ch.psi.daq.domain.query.DAQQueries;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.backend.BackendQuery;
import ch.psi.daq.domain.query.channels.ChannelConfigurationsRequest;
import ch.psi.daq.domain.query.channels.ChannelsRequest;
import ch.psi.daq.queryrest.config.QueryRestConfig;
import ch.psi.daq.queryrest.response.AbstractHTTPResponse;
import ch.psi.daq.queryrest.response.ResponseStreamWriter;
import ch.psi.daq.queryrest.response.formatter.AnyResponseFormatter;
import ch.psi.daq.queryrest.response.formatter.DAQConfigQueryResponseFormatter;
import ch.psi.daq.queryrest.response.formatter.DAQQueriesResponseFormatter;

/**
 * Takes a Java 8 stream and writes it to the output stream provided by the {@link ServletResponse}
 * of the current request.
 */
public abstract class AbstractResponseStreamWriter implements ResponseStreamWriter {
   private static final Logger LOGGER = LoggerFactory.getLogger(AbstractResponseStreamWriter.class);

   private Backend backend;

   protected void init(final Backend backend) {
      this.backend = backend;
   }

   @SuppressWarnings("unchecked")
   @Override
   public <R> void respond(
         final Object query,
         final R result,
         final OutputStream out,
         final AbstractHTTPResponse response) throws Exception {
      if (query instanceof DAQQueries) {
         backend.getApplicationContext()
               .getBean(QueryRestConfig.BEAN_NAME_FORMATTER_DAQ_QUERIES, DAQQueriesResponseFormatter.class)
               .format(
                     getJsonFactory(),
                     getObjectMapper(),
                     (List<Entry<DAQQueryElement, Stream<Quadruple<BackendQuery, ChannelName, ?, ?>>>>) result,
                     out,
                     response);
      } else if (query instanceof DAQConfigQuery) {
         backend.getApplicationContext()
               .getBean(QueryRestConfig.BEAN_NAME_FORMATTER_DAQ_CONFIG_QUERY, DAQConfigQueryResponseFormatter.class)
               .format(
                     getJsonFactory(),
                     getObjectMapper(),
                     (Entry<DAQConfigQueryElement, Stream<Triple<BackendQuery, ChannelName, ?>>>) result,
                     out,
                     response);
      } else if (query instanceof ChannelConfigurationsRequest) {
         backend.getApplicationContext()
               .getBean(QueryRestConfig.BEAN_NAME_FORMATTER_HISTORIC_CHANNEL_CONFIGURATION, AnyResponseFormatter.class)
               .format(
                     getJsonFactory(),
                     getObjectMapper(),
                     result,
                     out,
                     response);
      } else if (query instanceof ChannelName) {
         backend.getApplicationContext()
               .getBean(QueryRestConfig.BEAN_NAME_FORMATTER_HISTORIC_CHANNEL_CONFIGURATION, AnyResponseFormatter.class)
               .format(
                     getJsonFactory(),
                     getObjectMapper(),
                     result,
                     out,
                     response);

      } else if (query instanceof ChannelsRequest) {
         backend.getApplicationContext()
               .getBean(QueryRestConfig.BEAN_NAME_FORMATTER_ANY, AnyResponseFormatter.class)
               .format(
                     getJsonFactory(),
                     getObjectMapper(),
                     result,
                     out,
                     response);
      } else {
         final String message = String.format("'%s' has no response type for '%s'.", this.getClass(), query);
         LOGGER.error(message);
         throw new IllegalStateException(message);
      }
   }

   protected abstract JsonFactory getJsonFactory();

   protected abstract ObjectMapper getObjectMapper();
}
