package ch.psi.daq.queryrest.response.json;

import java.io.OutputStream;

import javax.servlet.http.HttpServletResponse;

import org.springframework.context.ApplicationContext;
import org.springframework.http.MediaType;

import com.hazelcast.util.collection.ArrayUtils;

import ch.psi.daq.domain.FieldNames;
import ch.psi.daq.domain.query.DAQQueries;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.operation.Compression;
import ch.psi.daq.domain.query.operation.EventField;
import ch.psi.daq.domain.query.response.ResponseFormat;
import ch.psi.daq.queryrest.response.AbstractHTTPResponse;
import ch.psi.daq.queryrest.response.ResponseStreamWriter;

public class JSONHTTPResponse extends AbstractHTTPResponse {
   public static final String FORMAT = "json";
   public static final String CONTENT_TYPE = MediaType.APPLICATION_JSON_VALUE;

   public JSONHTTPResponse() {
      super(ResponseFormat.JSON);
   }

   public JSONHTTPResponse(final Compression compression) {
      this();
      setCompression(compression);
   }

   @Override
   public void validateQuery(final Object query) {
      JSONHTTPResponse.defaultQueryValidation(query);
   }

   // @Override
   // public void respond(final ApplicationContext context, final HttpServletResponse response,
   // final Object query,
   // final Object result) throws Exception {
   // final OutputStream out = handleCompressionAndResponseHeaders(response, CONTENT_TYPE);
   // final boolean useTableFormat = JSONHTTPResponse.useTableFormat(query);
   //
   // final ResponseStreamWriter streamWriter;
   // if (useTableFormat) {
   // streamWriter = context.getBean(JSONTableResponseStreamWriter.class);
   // } else {
   // streamWriter = context.getBean(JSONResponseStreamWriter.class);
   // }
   //
   // // write the response back to the client using java 8 streams
   // streamWriter.respond(query, result, out, this);
   // }

   @Override
   public <R> void respond(
         final ApplicationContext context, 
         final HttpServletResponse response, 
         final Object query,
         final R result) throws Exception {
      final OutputStream out = handleCompressionAndResponseHeaders(response, CONTENT_TYPE);

      final ResponseStreamWriter streamWriter = context.getBean(JSONResponseStreamWriter.class);

      // write the response back to the client using java 8 streams
      streamWriter.respond(query, result, out, this);
   }

   public static void defaultQueryValidation(final Object queryObj) {
      if (queryObj instanceof DAQQueries) {
         final DAQQueries queries = (DAQQueries) queryObj;
         for (final DAQQueryElement query : queries) {
            if (query.getMapping() != null) {
               if (!ArrayUtils.contains(query.getRequest().getColumns(), FieldNames.FIELD_GLOBAL_TIME)) {
                  query.addEventField(EventField.globalMillis);
               }
            }
         }
      }
   }
}
