package ch.psi.daq.queryrest.response.msgpack;

import java.io.OutputStream;

import javax.servlet.http.HttpServletResponse;

import org.springframework.context.ApplicationContext;
import org.springframework.http.MediaType;

import ch.psi.daq.domain.query.operation.Compression;
import ch.psi.daq.domain.query.response.ResponseFormat;
import ch.psi.daq.queryrest.response.AbstractHTTPResponse;
import ch.psi.daq.queryrest.response.ResponseStreamWriter;
import ch.psi.daq.queryrest.response.json.JSONHTTPResponse;

public class MsgPackHTTPResponse extends AbstractHTTPResponse {
   public static final String FORMAT = "msgp";
   public static final String CONTENT_TYPE = MediaType.APPLICATION_OCTET_STREAM_VALUE;

   public MsgPackHTTPResponse() {
      super(ResponseFormat.MSGP);
   }

   public MsgPackHTTPResponse(final Compression compression) {
      this();
      setCompression(compression);
   }

   @Override
   public void validateQuery(final Object query){
      JSONHTTPResponse.defaultQueryValidation(query);
   }
   
   @Override
   public <R> void respond(
         final ApplicationContext context, 
         final HttpServletResponse response, 
         final Object query,
         final R result) throws Exception {
      final OutputStream out = handleCompressionAndResponseHeaders(response, CONTENT_TYPE);

      final ResponseStreamWriter streamWriter = context.getBean(MsgPackResponseStreamWriter.class);
      
      // write the response back to the client using java 8 streams
      streamWriter.respond(query, result, out, this);
   }
}
