package ch.psi.daq.queryrest.response.raw;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import ch.psi.daq.queryrest.response.raw.event.serialize.ConfigValuesHolder;
import ch.psi.daq.queryrest.response.raw.serialize.RawDeserializer;

public class RawDeserializerProvider<C, V> {
   private Map<Short, RawDeserializer<C, V>> deserializers = new HashMap<>();

   public boolean hasDeserializer(short id) {
      return deserializers.containsKey(id);
   }

   public boolean hasAllDeserializer(final Collection<Short> ids) {
      return deserializers.keySet().containsAll(ids);
   }

   public Set<Short> getDeserializerIds() {
      return deserializers.keySet();
   }

   public RawDeserializer<C, V> getDeserializer(short id) {
      return deserializers.get(id);
   }

   public void init(final ConfigValuesHolder<C, V> config) {
      for (final RawDeserializer<C, V> deserializer : deserializers.values()) {
         deserializer.init(config);
      }
   }

   public void setDeserializers(Map<Short, RawDeserializer<C, V>> deserializers) {
      this.deserializers = deserializers;
   }

   public Map<Short, RawDeserializer<C, V>> getDeserializers() {
      return deserializers;
   }

   public void add(RawDeserializer<C, V> deserializer) {
      this.deserializers.put(deserializer.getId(), deserializer);
   }
}
