package ch.psi.daq.queryrest.response.raw.event;

import java.io.OutputStream;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.psi.bsread.common.allocator.ByteBufferAllocator;
import ch.psi.bsread.message.Type;
import ch.psi.daq.domain.DataEvent;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.events.ChannelConfiguration;
import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.queryrest.response.raw.AbstractRawResponseFormatter;
import ch.psi.daq.queryrest.response.raw.event.serialize.EventConfig;
import ch.psi.daq.queryrest.response.raw.event.serialize.EventDeSerializer;
import ch.psi.daq.queryrest.response.raw.event.serialize.EventHeader;
import ch.psi.daq.queryrest.response.raw.event.serialize.EventHeaderDeSerializer;

public class RawEventResponseFormatter extends AbstractRawResponseFormatter implements ApplicationContextAware {
   private ObjectMapper mapper;
   private EventDeSerializer eventsDeSerializer;

   @Override
   public void setApplicationContext(ApplicationContext context) throws BeansException {
      mapper = context.getBean(DomainConfig.BEAN_NAME_OBJECT_MAPPER, ObjectMapper.class);
   }

   @Override
   protected void writeConfigs(final OutputStream os, final Map<ChannelName, DataEvent> latestEvents,
         final Pair<Map<ChannelName, ChannelConfiguration>, Map<ChannelName, DataEvent>> pair) {
      final EventHeaderDeSerializer eventHeaderDeSerializer =
            new EventHeaderDeSerializer(ByteBufferAllocator.DEFAULT_ALLOCATOR, mapper);
      eventHeaderDeSerializer.configure(null, pair.getKey());

      eventsDeSerializer = new EventDeSerializer(ByteBufferAllocator.DEFAULT_ALLOCATOR);
      final EventHeader eventHeader = new EventHeader();

      ChannelConfiguration config;
      DataEvent latestEvent;

      for (final Entry<ChannelName, ChannelConfiguration> entry : pair.getKey().entrySet()) {
         config = entry.getValue();
         latestEvent = latestEvents.get(entry.getKey());

         final EventConfig eventConfig = new EventConfig(
               EventConfig.BLOB_ID_UNDEFINED,
               config.getChannel(),
               config.getBackend().getName(),
               Type.newInstance(config.getType()),
               config.getShape(),
               config.getModulo(),
               config.getOffset(),
               EventConfig.getEncoding(getEncoding(latestEvent)),
               getCompression(latestEvent),
               config.getPulseId(),
               config.getGlobalTime().getUnscaled(),
               config.getSource(),
               config.getPrecision(),
               config.getUnit(),
               config.getDescription());

         eventHeader.addChannel(eventConfig);
      }

      // eventHeader.addDataConverterId(eventHeaderDeSerializer.getId());
      eventsDeSerializer.configure(eventHeader, pair.getKey());

      eventHeaderDeSerializer.serialize(os, eventHeader);
   }

   @Override
   protected void writeEvents(final OutputStream os,
         final Pair<Map<ChannelName, ChannelConfiguration>, Map<ChannelName, DataEvent>> pair) {
      eventsDeSerializer.serialize(os, pair.getRight());
   }
}
