package ch.psi.daq.queryrest.response.raw.event;

import java.io.OutputStream;
import java.util.List;
import java.util.Map.Entry;
import java.util.stream.Stream;

import javax.servlet.ServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import ch.psi.daq.common.tuple.Quadruple;
import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.domain.query.DAQQueries;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.backend.BackendQuery;
import ch.psi.daq.queryrest.config.QueryRestConfig;
import ch.psi.daq.queryrest.response.AbstractHTTPResponse;
import ch.psi.daq.queryrest.response.ResponseStreamWriter;

/**
 * Takes a Java 8 stream and writes it to the output stream provided by the {@link ServletResponse}
 * of the current request.
 */
public class RawEventResponseStreamWriter implements ResponseStreamWriter, ApplicationContextAware {
   private static final Logger LOGGER = LoggerFactory.getLogger(RawEventResponseStreamWriter.class);

   private ApplicationContext context;

   @Override
   public void setApplicationContext(ApplicationContext context) throws BeansException {
      final Backend backend = context.getBean(DomainConfig.BEAN_NAME_BACKEND_DEFAULT, Backend.class);
      this.context = backend.getApplicationContext();
   }

   @SuppressWarnings("unchecked")
   @Override
   public <R> void respond(
         final Object query,
         final R result,
         final OutputStream out,
         final AbstractHTTPResponse response) throws Exception {
      if (query instanceof DAQQueries) {
         context
               .getBean(QueryRestConfig.BEAN_NAME_FORMATTER_RAW_EVENT, RawEventResponseFormatter.class)
               .format(
                     null,
                     null,
                     (List<Entry<DAQQueryElement, Stream<Quadruple<BackendQuery, ChannelName, ?, ?>>>>) result,
                     out,
                     response);
      } else {
         final String message = String.format("'%s' has no response type for '%s'.", query);
         LOGGER.error(message);
         throw new IllegalStateException(message);
      }
   }
}
