package ch.psi.daq.queryrest.response.raw.event.serialize;

import java.util.Map;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import ch.psi.bsread.compression.Compression;
import ch.psi.bsread.message.Type;

@JsonInclude(Include.NON_DEFAULT)
public class EventConfig extends ch.psi.bsread.message.ChannelConfig {
   private static final long serialVersionUID = -8184893719772535221L;
   
   public static final short BLOB_ID_UNDEFINED = Short.MIN_VALUE;

   private short blobId = BLOB_ID_UNDEFINED;
   private String backend;
   private long pulseId;
   private long globalTime;
   private String source;
   private short precision;
   private String unit;
   private String description;
   private Map<Object, Object> valueMapping;

   public EventConfig() {}

   public EventConfig(short blobId, String name, String backend, Type type) {
      super(name, type);

      this.blobId = blobId;
      this.backend = backend;
   }

   public EventConfig(short blobId, String name, String backend, Type type, int modulo, int offset) {
      this(blobId, name, backend, type);

      this.setModulo(modulo);
      this.setOffset(offset);
   }

   public EventConfig(short blobId, String name, String backend, Type type, int[] shape, int modulo, int offset) {
      this(blobId, name, backend, type, modulo, offset);

      this.setShape(shape);
   }

   public EventConfig(short blobId, String name, String backend, Type type, int[] shape, int modulo, int offset,
         String encoding) {
      this(blobId, name, backend, type, shape, modulo, offset);

      this.setEncoding(encoding);
   }

   public EventConfig(short blobId, String name, String backend, Type type, int[] shape, int modulo, int offset, String encoding,
         Compression compression) {
      this(blobId, name, backend, type, shape, modulo, offset, encoding);

      this.setCompression(compression);
   }

   public EventConfig(short blobId, String name, String backend, Type type, int[] shape, int modulo, int offset, String encoding,
         Compression compression, long pulseId, long globalTime, String source, short precision, String unit,
         String description) {
      this(blobId, name, backend, type, shape, modulo, offset, encoding, compression);

      setPulseId(pulseId);
      setGlobalTime(globalTime);
      setSource(source);
      setPrecision(precision);
      setUnit(unit);
      setDescription(description);
   }

   public EventConfig(EventConfig other) {
      super(other);

      this.blobId = other.blobId;
      this.backend = other.backend;
      this.pulseId = other.pulseId;
      this.globalTime = other.globalTime;
      this.source = other.source;
      this.precision = other.precision;
      this.unit = other.unit;
      this.description = other.description;
      this.valueMapping = other.valueMapping;
   }

   public void copy(EventConfig other) {
      super.copy(other);

      this.blobId = other.blobId;
      this.backend = other.backend;
      this.pulseId = other.pulseId;
      this.globalTime = other.globalTime;
      this.source = other.source;
      this.precision = other.precision;
      this.unit = other.unit;
      this.description = other.description;
      this.valueMapping = other.valueMapping;
   }
   
   public short getBlobId() {
      return blobId;
   }

   public void setBlobId(short blobId) {
      this.blobId = blobId;
   }

   public String getBackend() {
      return backend;
   }

   public void setBackend(String backend) {
      this.backend = backend;
   }

   public long getPulseId() {
      return pulseId;
   }

   public void setPulseId(long pulseId) {
      this.pulseId = pulseId;
   }

   public long getGlobalTime() {
      return globalTime;
   }

   public void setGlobalTime(long globalTime) {
      this.globalTime = globalTime;
   }

   public String getSource() {
      return source;
   }

   public void setSource(String source) {
      this.source = source;
   }

   public short getPrecision() {
      return precision;
   }

   public void setPrecision(short precision) {
      this.precision = precision;
   }

   public String getUnit() {
      return unit;
   }

   public void setUnit(String unit) {
      this.unit = unit;
   }

   public String getDescription() {
      return description;
   }

   public void setDescription(String description) {
      this.description = description;
   }
   
   public Map<Object, Object> getValueMapping() {
      return valueMapping;
   }

   public void setValueMapping(Map<Object, Object> valueMapping) {
      this.valueMapping = valueMapping;
   }

   @Override
   public int hashCode() {
      final int prime = 31;
      int result = super.hashCode();
      result = prime * result + ((backend == null) ? 0 : backend.hashCode());
      result = prime * result + blobId;
      result = prime * result + ((description == null) ? 0 : description.hashCode());
      result = prime * result + precision;
      result = prime * result + (int) (pulseId ^ (pulseId >>> 32));
      result = prime * result + ((source == null) ? 0 : source.hashCode());
      result = prime * result + (int) (globalTime ^ (globalTime >>> 32));
      result = prime * result + ((unit == null) ? 0 : unit.hashCode());
      result = prime * result + ((valueMapping == null) ? 0 : valueMapping.hashCode());
      return result;
   }

   @Override
   public boolean equals(Object obj) {
      if (this == obj)
         return true;
      if (!super.equals(obj))
         return false;
      if (getClass() != obj.getClass())
         return false;
      EventConfig other = (EventConfig) obj;
      if (backend == null) {
         if (other.backend != null)
            return false;
      } else if (!backend.equals(other.backend))
         return false;
      if (blobId != other.blobId)
         return false;
      if (description == null) {
         if (other.description != null)
            return false;
      } else if (!description.equals(other.description))
         return false;
      if (precision != other.precision)
         return false;
      if (pulseId != other.pulseId)
         return false;
      if (source == null) {
         if (other.source != null)
            return false;
      } else if (!source.equals(other.source))
         return false;
      if (globalTime != other.globalTime)
         return false;
      if (unit == null) {
         if (other.unit != null)
            return false;
      } else if (!unit.equals(other.unit))
         return false;
      if (valueMapping == null) {
         if (other.valueMapping != null)
            return false;
      } else if (!valueMapping.equals(other.valueMapping))
         return false;
      return true;
   }
}
