package ch.psi.daq.queryrest.response.raw.event.serialize;

import java.io.Serializable;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.json.BackendChannel;
import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.queryrest.response.raw.IdProvider;
import ch.psi.daq.queryrest.response.raw.Ids;

@JsonInclude(Include.NON_DEFAULT)
public class EventHeader implements IdProvider, Serializable {
   private static final long serialVersionUID = -5342463617678945523L;

   public static final short ID = Ids.DATA_EVENTS_HEADER;

   private Set<Short> eventSerializerIds = new TreeSet<>();
   private Map<BackendChannel, EventConfig> channelsMapping = new LinkedHashMap<>();

   public EventHeader() {}

   @JsonIgnore
   public short getId() {
      return ID;
   }

   public Set<Short> getEventSerializerIds() {
      return eventSerializerIds;
   }

   public void setEventSerializerIds(Set<Short> eventSerializerIds) {
      this.eventSerializerIds = eventSerializerIds;
   }

   public void addEventSerializerId(short eventSerializerId) {
      this.eventSerializerIds.add(eventSerializerId);
   }

   public Collection<EventConfig> getChannels() {
      return channelsMapping.values();
   }

   public void setChannels(Collection<EventConfig> channels) {
      this.channelsMapping.clear();
      for (EventConfig channel : channels) {
         this.channelsMapping.put(
               new ChannelName(channel.getName(), Backend.byName(channel.getBackend())),
               channel);
      }
   }

   public void addChannel(EventConfig channel) {
      this.channelsMapping.put(
            new ChannelName(channel.getName(), Backend.byName(channel.getBackend())),
            channel);
   }

   @JsonIgnore
   public Map<BackendChannel, EventConfig> getChannelsMapping() {
      return channelsMapping;
   }
}
