package ch.psi.daq.queryrest.response.raw.receive;

import java.io.Closeable;
import java.util.Collection;
import java.util.Map;
import java.util.function.Consumer;

import ch.psi.daq.domain.json.BackendChannel;

public interface IReceiver<C, V> extends Closeable {

   /**
    * Stop receiving and close resources.
    */
   void close();

   /**
    * Establishes the connection.
    */
   void connect();

   /**
    * Receive the next message (blocks for the next).
    * 
    * @return Message The next message or <tt>null</tt> for termination.
    * @throws RuntimeException Might throw a RuntimeException
    */
   Message<C, V> receive() throws RuntimeException;

   /**
    * Provides access to the Value change handlers
    * 
    * @return Collection The handlers
    */
   Collection<Consumer<Map<BackendChannel, V>>> getValueHandlers();

   /**
    * Provides access to the Config change handler
    * 
    * @return Collection The handlers
    */
   Collection<Consumer<C>> getConfigHandlers();
}
