package ch.psi.daq.queryrest.response.raw.serialize;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.psi.bsread.common.allocator.ByteBufferAllocator;
import ch.psi.bsread.common.helper.ByteBufferHelper;
import ch.psi.daq.queryrest.response.raw.IdProvider;
import ch.psi.daq.queryrest.response.raw.event.serialize.ConfigValuesHolder;

public interface RawDeserializer<C, V> extends IdProvider {
   public static final Logger _LOGGER = LoggerFactory.getLogger(RawDeserializer.class);
   public static final ByteOrder DEFAULT_ENCODING = RawSerializer.DEFAULT_ENCODING;

   public static ByteBuffer readBytes(final InputStream is) throws IOException {
      final ByteBuffer sizeBuf = ByteBufferAllocator.DEFAULT_ALLOCATOR
            .apply(Long.BYTES)
            .order(RawDeserializer.DEFAULT_ENCODING);
      ByteBufferHelper.readByteBuffer(is, sizeBuf);
      long size = sizeBuf.getLong(0);

      final ByteBuffer blob = ByteBufferAllocator.DEFAULT_ALLOCATOR
            .apply((int) size)
            .order(RawDeserializer.DEFAULT_ENCODING);
      ByteBufferHelper.readByteBuffer(is, blob);
      return blob;
   }

   public static short readDataId(final InputStream is) throws IOException {
      final ByteBuffer idBuf = ByteBufferAllocator.DEFAULT_ALLOCATOR
            .apply(Short.BYTES)
            .order(RawDeserializer.DEFAULT_ENCODING);
      ByteBufferHelper.readByteBuffer(is, idBuf);
      return getDataId(idBuf);
   }

   public static short getDataId(final ByteBuffer blob) {
      return blob.getShort(0);
   }

   void init(final ConfigValuesHolder<C, V> config);
   
   void deserialize(final ByteBuffer buffer) throws RuntimeException;
}
