package ch.psi.daq.test.queryrest.config;

import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;

import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.Lazy;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;
import org.springframework.context.annotation.Scope;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;

import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.backend.BackendType;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.events.ChannelEvent;
import ch.psi.daq.domain.query.processor.QueryProcessor;
import ch.psi.daq.domain.query.processor.QueryProcessorLocal;
import ch.psi.daq.domain.reader.StreamEventReader;
import ch.psi.daq.domain.test.reader.TestReader;
import ch.psi.daq.query.config.QueryConfig;
import ch.psi.daq.queryrest.config.QueryRestConfig;

@Configuration
@ComponentScan
@EnableWebMvc
@PropertySources(value = {
      @PropertySource(value = {"classpath:queryrest-test.properties"})
})
public class DaqWebMvcConfig extends WebMvcConfigurerAdapter {

   // ensure that properties in dispatcher.properties are loaded first and then overwritten by the
   // properties in dispatcher-test.properties
   @Import(value = {QueryRestConfig.class})
   static class InnerConfiguration {
   }
   
   // somehow needed to make sure @Import elements will get initialized and afterPropertiesSet will
   // get called (ensuring BackendAcces gets initialized according hierarchy)
   @Resource
   private QueryRestConfig queryRestConfig;

   public static final String IS_INITIAL_CHANNELS = "initial.channels";
   public static final String BEAN_NAME_READER = "dummyReader";

   @Resource
   private ApplicationContext context;

   @SuppressWarnings("unchecked")
   @PostConstruct
   public void afterPropertiesSet() {
      // init BackendAccesses
      final List<Backend> backends = context.getBean(DomainConfig.BEAN_NAME_BACKENDS, List.class);
      final boolean overload = true;
      for (final Backend backend : backends) {
         final BackendType backendType = backend.getType();

         // backendType.initBean(backend, BEAN_NAME_READER, DataReader.class, overload);
         backendType.initBean(backend, BEAN_NAME_READER, StreamEventReader.class, overload, backend);
      }
   }

   @Bean(name = BEAN_NAME_READER)
   @Scope(BeanDefinition.SCOPE_PROTOTYPE)
   @Lazy
   public StreamEventReader<ChannelEvent> testReader(final Backend backend) {
      return new TestReader(backend);
   }

   // make sure we use a local QueryProcessor even for distributed calls -> no Hazelcast needs to be
   // started
   @Bean(name = QueryConfig.BEAN_NAME_QUERY_PROCESSOR_DISTRIBUTED)
   @Lazy
   public QueryProcessor distributedQueryProcessor() {
      return new QueryProcessorLocal();
   }

   @Override
   public void configureMessageConverters(List<HttpMessageConverter<?>> converters) {
      /**
       * This is necessary so that the message conversion uses the configured object mapper.
       * Otherwise, a separate object mapper is instantiated for Springs message conversion.
       */
      final MappingJackson2HttpMessageConverter converter =
            context.getBean(DomainConfig.BEAN_NAME_MESSAGE_CONVERTER, MappingJackson2HttpMessageConverter.class);
      converters.add(converter);
      super.configureMessageConverters(converters);
   }
}
