package ch.psi.daq.test.queryrest.controller;

import static org.junit.Assert.assertEquals;

import java.util.List;
import java.util.stream.Collectors;

import javax.annotation.Resource;

import org.junit.After;
import org.junit.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.psi.daq.common.time.Time;
import ch.psi.daq.domain.DataEvent;
import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.json.ChannelEventTableImpl;
import ch.psi.daq.domain.query.DAQQuery;
import ch.psi.daq.domain.query.mapping.Mapping;
import ch.psi.daq.domain.query.operation.EventField;
import ch.psi.daq.domain.query.response.Response;
import ch.psi.daq.domain.request.range.RequestRangePulseId;
import ch.psi.daq.test.queryrest.AbstractDaqRestTest;

/**
 * Tests the {@link DaqController} implementation.
 */
public abstract class AbstractQueryRestControllerTableTest extends AbstractDaqRestTest {

   public static final String TEST_CHANNEL_01 = "testChannel1";
   public static final String TEST_CHANNEL_02 = "testChannel2";
   public static final String TEST_CHANNEL_WAVEFORM_01 = "testChannelWaveform1";
   public static final String TEST_CHANNEL_WAVEFORM_02 = "testChannelWaveform2";
   public static final String[] TEST_CHANNEL_NAMES = new String[] {TEST_CHANNEL_01, TEST_CHANNEL_02};

   @Resource(name = DomainConfig.BEAN_NAME_BACKEND_DEFAULT)
   private Backend backend;

   @After
   public void tearDown() throws Exception {}
   
   protected abstract ObjectMapper getResponseMapper();
   
   protected abstract Response getResponse();

   @Test
   public void testPulseRangeQuery() throws Exception {
      DAQQuery request = new DAQQuery(
            new RequestRangePulseId(
                  100,
                  101),
            TEST_CHANNEL_NAMES);
      request.setMapping(new Mapping());
      request.addEventField(EventField.pulseId);
      request.addEventField(EventField.globalSeconds);
      request.addEventField(EventField.globalMillis);
      request.addEventField(EventField.iocSeconds);
      request.addEventField(EventField.iocMillis);
      request.addEventField(EventField.value);
      request.setResponse(getResponse());

      String content = mapper.writeValueAsString(request);
      System.out.println(content);
      
      MvcResult result = this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(DomainConfig.PATH_QUERY)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andReturn();

      byte[] responseBytes = result.getResponse().getContentAsByteArray();
      System.out.println("ResponseSize: "+responseBytes.length);
      ChannelEventTableImpl table = getResponseMapper().readValue(responseBytes, ChannelEventTableImpl.class);
      
      assertEquals(2, table.size());
      List<DataEvent> events = table.getEventsOfRow(0).collect(Collectors.toList());
      assertEquals(2, events.size());
      DataEvent event = events.get(0);
      assertEquals(TEST_CHANNEL_01, event.getChannel());
      assertEquals(backend, event.getBackend());
      assertEquals(100, event.getPulseId());
      assertEquals(new Time(1, 0), event.getGlobalTime());
      assertEquals(new Time(1, 0), event.getIocTime());
      assertEquals(100, event.getValue(Number.class).longValue());
      event = events.get(1);
      assertEquals(TEST_CHANNEL_02, event.getChannel());
      assertEquals(backend, event.getBackend());
      assertEquals(100, event.getPulseId());
      assertEquals(new Time(1, 0), event.getGlobalTime());
      assertEquals(new Time(1, 0), event.getIocTime());
      assertEquals(100, event.getValue(Number.class).longValue());
      
      events = table.getEventsOfRow(1).collect(Collectors.toList());
      assertEquals(2, events.size());
      event = events.get(0);
      assertEquals(TEST_CHANNEL_01, event.getChannel());
      assertEquals(backend, event.getBackend());
      assertEquals(101, event.getPulseId());
      assertEquals(new Time(1, 10000000), event.getGlobalTime());
      assertEquals(new Time(1, 10000000), event.getIocTime());
      assertEquals(101, event.getValue(Number.class).longValue());
      event = events.get(1);
      assertEquals(TEST_CHANNEL_02, event.getChannel());
      assertEquals(backend, event.getBackend());
      assertEquals(101, event.getPulseId());
      assertEquals(new Time(1, 10000000), event.getGlobalTime());
      assertEquals(new Time(1, 10000000), event.getIocTime());
      assertEquals(101, event.getValue(Number.class).longValue());
   }
}
