package ch.psi.daq.test.queryrest.controller;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.List;
import java.util.stream.Collectors;

import javax.annotation.Resource;

import org.junit.After;
import org.junit.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import ch.psi.bsread.message.Type;
import ch.psi.daq.common.ordering.Ordering;
import ch.psi.daq.common.time.TimeUtils;
import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.events.ChannelConfiguration;
import ch.psi.daq.domain.events.impl.ChannelConfigurationImpl;
import ch.psi.daq.domain.json.ChannelConfigurations;
import ch.psi.daq.domain.json.ChannelConfigurationsList;
import ch.psi.daq.domain.query.DAQConfigQuery;
import ch.psi.daq.domain.query.channels.ChannelConfigurationsRequest;
import ch.psi.daq.domain.query.channels.ChannelConfigurationsResponse;
import ch.psi.daq.domain.query.channels.LongHash;
import ch.psi.daq.domain.request.range.RequestRangePulseId;
import ch.psi.daq.test.queryrest.AbstractDaqRestTest;

/**
 * Tests the {@link DaqController} implementation.
 */
public class QueryRestControllerChannelConfigurationTest extends AbstractDaqRestTest {

   @Resource(name = DomainConfig.BEAN_NAME_BACKEND_DEFAULT)
   private Backend backend;

   private ObjectMapper objectMapper = new ObjectMapper();

   @After
   public void tearDown() throws Exception {}

   @Test
   public void testChannelConfigurationsHash() throws Exception {
      MvcResult result = this.mockMvc
            .perform(
                  MockMvcRequestBuilders
                        .get(DomainConfig.PATH_CHANNELS_CONFIG_HASH)
                        .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andReturn();

      String response = result.getResponse().getContentAsString();
      System.out.println("Response: " + response);
      
      LongHash hash1 =
            objectMapper.readValue(response, LongHash.class);
      
      result = this.mockMvc
            .perform(
                  MockMvcRequestBuilders
                        .get(DomainConfig.PATH_CHANNELS_CONFIG_HASH)
                        .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andReturn();

      response = result.getResponse().getContentAsString();
      System.out.println("Response: " + response);
      
      LongHash hash2 =
            objectMapper.readValue(response, LongHash.class);
      
      assertEquals(hash1.getHash(), hash2.getHash());
   }
   
   @Test
   public void testChannelConfigurationQuery_01() throws Exception {
      DAQConfigQuery query = new DAQConfigQuery(
            new RequestRangePulseId(
                  100,
                  101),
            backend.getName() + "1", backend.getName() + "2");

      String content = mapper.writeValueAsString(query);
      System.out.println(content);

      MvcResult result = this.mockMvc.perform(MockMvcRequestBuilders
            .post(DomainConfig.PATH_QUERY_CONFIG)
            .contentType(MediaType.APPLICATION_JSON)
            .content(content))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andReturn();

      String response = result.getResponse().getContentAsString();
      System.out.println("Response: " + response);

      // test conversion used in DAQProcessing
      ChannelConfigurationsList channelConfigurations =
            objectMapper.readValue(response, ChannelConfigurationsList.class);
      assertEquals(2, channelConfigurations.size());
      ChannelConfigurations configs = channelConfigurations.get(0);
      assertEquals(backend.getName() + "1", configs.getChannel().getName());
      assertEquals(backend, configs.getChannel().getBackend());
      List<ChannelConfiguration> configsList = configs.getConfigs().collect(Collectors.toList());
      assertEquals(2, configsList.size());
      ChannelConfiguration config = configsList.get(0);
      assertEquals(backend.getName() + "1", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getStartPulseId() * 10, 0), config.getGlobalTime());
      assertEquals(query.getRange().getStartPulseId(), config.getPulseId());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertEquals(Type.Int32.getKey(), config.getType());
      assertEquals("unknown", config.getSource());
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
      config = configsList.get(1);
      assertEquals(backend.getName() + "1", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getEndPulseId() * 10, 0), config.getGlobalTime());
      assertEquals(query.getRange().getEndPulseId(), config.getPulseId());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertEquals(Type.Int32.getKey(), config.getType());
      assertEquals("unknown", config.getSource());
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());

      configs = channelConfigurations.get(1);
      assertEquals(backend.getName() + "2", configs.getChannel().getName());
      assertEquals(backend, configs.getChannel().getBackend());
      configsList = configs.getConfigs().collect(Collectors.toList());
      assertEquals(2, configsList.size());
      config = configsList.get(0);
      assertEquals(backend.getName() + "2", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getStartPulseId() * 10, 0), config.getGlobalTime());
      assertEquals(query.getRange().getStartPulseId(), config.getPulseId());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertEquals(Type.Int32.getKey(), config.getType());
      assertEquals("unknown", config.getSource());
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
      config = configsList.get(1);
      assertEquals(backend.getName() + "2", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getEndPulseId() * 10, 0), config.getGlobalTime());
      assertEquals(query.getRange().getEndPulseId(), config.getPulseId());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertEquals(Type.Int32.getKey(), config.getType());
      assertEquals("unknown", config.getSource());
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());

      // test if backend and channel/name not set
      this.mockMvc.perform(MockMvcRequestBuilders
            .post(DomainConfig.PATH_QUERY_CONFIG)
            .contentType(MediaType.APPLICATION_JSON)
            .content(content))
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.name").value(backend.getName() + "1"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[0].name").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[0].backend").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[0].type").value(config.getType()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[0].shape[0]").value(config.getShape()[0]))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[1].name").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[1].backend").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[1].type").value(config.getType()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[1].shape[0]").value(config.getShape()[0]))            .andExpect(MockMvcResultMatchers.jsonPath("$[0].configs[2]").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[0].name").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[0].backend").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[0].type").value(config.getType()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[0].shape[0]").value(config.getShape()[0]))            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[1].name").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[1].backend").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[1].type").value(config.getType()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[1].shape[0]").value(config.getShape()[0]))            .andExpect(MockMvcResultMatchers.jsonPath("$[1].configs[2]").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[2]").doesNotExist());
   }

   @Test
   public void testChannelConfigurationQuery_02() throws Exception {
      DAQConfigQuery query = new DAQConfigQuery(
            new RequestRangePulseId(
                  100,
                  101),
            backend.getName() + "1", backend.getName() + "2");
      query.setOrdering(Ordering.desc);

      String content = mapper.writeValueAsString(query);
      System.out.println(content);

      MvcResult result = this.mockMvc.perform(MockMvcRequestBuilders
            .post(DomainConfig.PATH_QUERY_CONFIG)
            .contentType(MediaType.APPLICATION_JSON)
            .content(content))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andReturn();

      String response = result.getResponse().getContentAsString();
      System.out.println("Response: " + response);

      // test conversion used in DAQProcessing
      ChannelConfigurationsList channelConfigurations =
            objectMapper.readValue(response, ChannelConfigurationsList.class);
      assertEquals(2, channelConfigurations.size());
      ChannelConfigurations configs = channelConfigurations.get(0);
      assertEquals(backend.getName() + "1", configs.getChannel().getName());
      assertEquals(backend, configs.getChannel().getBackend());
      List<ChannelConfiguration> configsList = configs.getConfigs().collect(Collectors.toList());
      assertEquals(2, configsList.size());
      ChannelConfiguration config = configsList.get(0);
      assertEquals(backend.getName() + "1", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getEndPulseId() * 10, 0), config.getGlobalTime());
      assertEquals(query.getRange().getEndPulseId(), config.getPulseId());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertEquals(Type.Int32.getKey(), config.getType());
      assertEquals("unknown", config.getSource());
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
      config = configsList.get(1);
      assertEquals(backend.getName() + "1", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getStartPulseId() * 10, 0), config.getGlobalTime());
      assertEquals(query.getRange().getStartPulseId(), config.getPulseId());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertEquals(Type.Int32.getKey(), config.getType());
      assertEquals("unknown", config.getSource());
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());

      configs = channelConfigurations.get(1);
      assertEquals(backend.getName() + "2", configs.getChannel().getName());
      assertEquals(backend, configs.getChannel().getBackend());
      configsList = configs.getConfigs().collect(Collectors.toList());
      assertEquals(2, configsList.size());
      config = configsList.get(0);
      assertEquals(backend.getName() + "2", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getEndPulseId() * 10, 0), config.getGlobalTime());
      assertEquals(query.getRange().getEndPulseId(), config.getPulseId());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertEquals(Type.Int32.getKey(), config.getType());
      assertEquals("unknown", config.getSource());
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
      config = configsList.get(1);
      assertEquals(backend.getName() + "2", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals(TimeUtils.getTimeFromMillis(query.getRange().getStartPulseId() * 10, 0), config.getGlobalTime());
      assertEquals(query.getRange().getStartPulseId(), config.getPulseId());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertEquals(Type.Int32.getKey(), config.getType());
      assertEquals("unknown", config.getSource());
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
   }

   @Test
   public void testChannelConfigurations_01() throws Exception {
      ChannelConfigurationsRequest request = new ChannelConfigurationsRequest(
            "int32");

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      MvcResult result = this.mockMvc.perform(MockMvcRequestBuilders
            .post(DomainConfig.PATH_CHANNELS_CONFIG)
            .contentType(MediaType.APPLICATION_JSON)
            .content(content))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andReturn();

      String response = result.getResponse().getContentAsString();
      System.out.println("Response: " + response);

      // test conversion used in DAQProcessing
      List<ChannelConfigurationsResponse> channelConfigurations =
            objectMapper.readValue(response, new TypeReference<List<ChannelConfigurationsResponse>>() {});
      assertEquals(3, channelConfigurations.size());
      ChannelConfigurationsResponse configResponse = channelConfigurations.get(0);
      assertEquals(backend, configResponse.getBackend());

      List<ChannelConfiguration> configs = configResponse.getChannels().collect(Collectors.toList());
      assertEquals(4, configs.size());
      ChannelConfiguration config = configs.get(0);
      assertEquals("Int32Scalar", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals("int32", config.getType());
      assertEquals("unknown", config.getSource());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertNull(config.getGlobalTime());
      assertTrue(config.getPulseId() == 0);
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
      config = configs.get(1);
      assertEquals("Int32Waveform", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals("int32", config.getType());
      assertEquals("unknown", config.getSource());
      assertArrayEquals(new int[] {8}, config.getShape());
      assertNull(config.getGlobalTime());
      assertTrue(config.getPulseId() == 0);
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
      config = configs.get(2);
      assertEquals("UInt32Scalar", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals("uint32", config.getType());
      assertEquals("unknown", config.getSource());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertNull(config.getGlobalTime());
      assertTrue(config.getPulseId() == 0);
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
      config = configs.get(3);
      assertEquals("UInt32Waveform", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals("uint32", config.getType());
      assertEquals("unknown", config.getSource());
      assertArrayEquals(new int[] {8}, config.getShape());
      assertNull(config.getGlobalTime());
      assertTrue(config.getPulseId() == 0);
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
   }

   @Test
   public void testChannelConfigurations_02() throws Exception {
      MvcResult result = this.mockMvc.perform(MockMvcRequestBuilders
            .get(DomainConfig.PATH_CHANNELS_CONFIG + "/int16")
            .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andReturn();

      String response = result.getResponse().getContentAsString();
      System.out.println("Response: " + response);

      // test conversion used in DAQProcessing
      List<ChannelConfigurationsResponse> channelConfigurations =
            objectMapper.readValue(response, new TypeReference<List<ChannelConfigurationsResponse>>() {});
      assertEquals(3, channelConfigurations.size());
      ChannelConfigurationsResponse configResponse = channelConfigurations.get(0);
      assertEquals(backend, configResponse.getBackend());

      List<ChannelConfiguration> configs = configResponse.getChannels().collect(Collectors.toList());
      assertEquals(4, configs.size());
      ChannelConfiguration config = configs.get(0);
      assertEquals("Int16Scalar", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals("int16", config.getType());
      assertEquals("unknown", config.getSource());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertNull(config.getGlobalTime());
      assertTrue(config.getPulseId() == 0);
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
      config = configs.get(1);
      assertEquals("Int16Waveform", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals("int16", config.getType());
      assertEquals("unknown", config.getSource());
      assertArrayEquals(new int[] {8}, config.getShape());
      assertNull(config.getGlobalTime());
      assertTrue(config.getPulseId() == 0);
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
      config = configs.get(2);
      assertEquals("UInt16Scalar", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals("uint16", config.getType());
      assertEquals("unknown", config.getSource());
      assertArrayEquals(new int[] {1}, config.getShape());
      assertNull(config.getGlobalTime());
      assertTrue(config.getPulseId() == 0);
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
      config = configs.get(3);
      assertEquals("UInt16Waveform", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals("uint16", config.getType());
      assertEquals("unknown", config.getSource());
      assertArrayEquals(new int[] {8}, config.getShape());
      assertNull(config.getGlobalTime());
      assertTrue(config.getPulseId() == 0);
      assertEquals(0, config.getModulo());
      assertEquals(0, config.getOffset());
   }

   @Test
   public void testChannelConfigurations_03() throws Exception {
      MvcResult result = this.mockMvc.perform(MockMvcRequestBuilders
            .get(DomainConfig.PATH_CHANNEL_CONFIG + "/Int16Waveform")
            .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andReturn();

      String response = result.getResponse().getContentAsString();
      System.out.println("Response: " + response);

      // test conversion used in DAQProcessing
      ChannelConfiguration config =
            objectMapper.readValue(response, ChannelConfigurationImpl.class);
      assertEquals("Int16Waveform", config.getChannel());
      assertEquals(backend, config.getBackend());
      assertEquals("int16", config.getType());
      assertEquals("unknown", config.getSource());
      assertArrayEquals(new int[] {8}, config.getShape());
   }

   @Test
   public void testChannelConfigurationsBackendOrder() throws Exception {
      ChannelConfigurationsRequest request = new ChannelConfigurationsRequest(
            "int64", Ordering.asc, backend);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(DomainConfig.PATH_CHANNELS_CONFIG)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0].name").value("Int64Scalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0].backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0].channel").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1].name").value("Int64Waveform"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1].backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1].channel").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2].name").value("UInt64Scalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2].backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0].channel").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3].name").value("UInt64Waveform"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3].backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0].channel").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[4]").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").doesNotExist());


      request = new ChannelConfigurationsRequest(
            "int64", Ordering.desc, backend);

      content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(DomainConfig.PATH_CHANNELS_CONFIG)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0].name").value("UInt64Waveform"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0].backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0].channel").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1].name").value("UInt64Scalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1].backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1].channel").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2].name").value("Int64Waveform"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2].backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2].channel").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3].name").value("Int64Scalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3].backend").value(backend.getName()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3].channel").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[4]").doesNotExist())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").doesNotExist());
   }
}
