package ch.psi.daq.test.queryrest.response;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;

import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.query.DAQQuery;
import ch.psi.daq.domain.query.operation.Compression;
import ch.psi.daq.domain.query.response.Response;
import ch.psi.daq.domain.request.range.RequestRangePulseId;
import ch.psi.daq.queryrest.response.csv.CSVHTTPResponse;
import ch.psi.daq.queryrest.response.json.JSONHTTPResponse;
import ch.psi.daq.test.queryrest.AbstractDaqRestTest;

public class ResponseQueryTest extends AbstractDaqRestTest {

   @Resource(name = DomainConfig.BEAN_NAME_BACKEND_DEFAULT)
   private Backend backend;
   private ObjectMapper mapper;

   @PostConstruct
   public void afterPropertiesSet() {
      mapper = backend.getApplicationContext().getBean(DomainConfig.BEAN_NAME_OBJECT_MAPPER, ObjectMapper.class);
   }

   @Test
   public void test_JSON_01() throws JsonParseException, JsonMappingException, IOException {
      Response respose = new CSVHTTPResponse();

      String value = mapper.writeValueAsString(respose);

      Response deserial = mapper.readValue(value, Response.class);

      assertEquals(respose.getClass(), deserial.getClass());
      assertEquals(respose.getFormat(), deserial.getFormat());
      assertEquals(respose.getCompression(), deserial.getCompression());
   }

   @Test
   public void test_JSON_02() throws JsonParseException, JsonMappingException, IOException {
      DAQQuery query = new DAQQuery(
            new RequestRangePulseId(
                  0,
                  100),
            "TestChannel_01");
      query.setResponse(new CSVHTTPResponse(Compression.GZIP));

      String value = mapper.writeValueAsString(query);

      DAQQuery deserial = mapper.readValue(value, DAQQuery.class);

      assertNotNull(deserial.getResponse());
      assertEquals(query.getResponse().getClass(), deserial.getResponse().getClass());
      assertEquals(query.getResponse().getFormat(), deserial.getResponse().getFormat());
      assertEquals(query.getResponse().getCompression(), deserial.getResponse().getCompression());

      assertEquals(query.getResponse().getFileSuffix(), deserial.getResponse().getFileSuffix());
   }

   @Test
   public void test_JSON_03() throws JsonParseException, JsonMappingException, IOException {
      DAQQuery query = new DAQQuery(
            new RequestRangePulseId(
                  0,
                  100),
            "TestChannel_01");
      query.setResponse(new JSONHTTPResponse(Compression.NONE));

      String value = mapper.writeValueAsString(query);

      int index = value.indexOf("format");
      assertTrue(index >= 0);
      index = value.indexOf("format", index + 1);
      // ensure string contains identifier only once
      assertEquals(-1, index);

      DAQQuery deserial = mapper.readValue(value, DAQQuery.class);

      assertNotNull(deserial.getResponse());
      assertEquals(query.getResponse().getClass(), deserial.getResponse().getClass());
      assertEquals(query.getResponse().getFormat(), deserial.getResponse().getFormat());
      assertEquals(query.getResponse().getCompression(), deserial.getResponse().getCompression());

      assertEquals(query.getResponse().getFormat().getFileSuffix(), deserial.getResponse().getFileSuffix());
   }

   @Test
   public void test_JSON_04() throws JsonParseException, JsonMappingException, IOException {
      DAQQuery query = new DAQQuery(
            new RequestRangePulseId(
                  0,
                  100),
            "TestChannel_01");

      String value = mapper.writeValueAsString(query);

      DAQQuery deserial = mapper.readValue(value, DAQQuery.class);

      assertNull(deserial.getResponse());
   }
}
